Module: PostHog::Utils
- Included in:
- Client, FeatureFlagsPoller, FieldParser, SendWorker, Transport
- Defined in:
- lib/posthog/utils.rb
Defined Under Namespace
Classes: SizeLimitedHash
Constant Summary collapse
- UTC_OFFSET_WITH_COLON =
'%s%02d:%02d'.freeze
- UTC_OFFSET_WITHOUT_COLON =
UTC_OFFSET_WITH_COLON.sub(':', '')
Class Method Summary collapse
- .convert_to_datetime(value) ⇒ Object
- .date_in_iso8601(date) ⇒ Object
- .datetime_in_iso8601(datetime) ⇒ Object
- .formatted_offset(time, colon = true, alternate_utc_string = nil) ⇒ Object
-
.is_valid_regex(regex) ⇒ Object
TODO: Rename to
valid_regex?in future version. -
.isoify_dates(hash) ⇒ Object
public: Returns a new hash with all the date values in the into iso8601 strings.
-
.isoify_dates!(hash) ⇒ Object
public: Converts all the date values in the into iso8601 strings in place.
- .seconds_to_utc_offset(seconds, colon = true) ⇒ Object
-
.stringify_keys(hash) ⇒ Object
public: Return a new hash with keys as strings.
-
.symbolize_keys(hash) ⇒ Object
public: Return a new hash with keys converted from strings to symbols.
-
.symbolize_keys!(hash) ⇒ Object
public: Convert hash keys from strings to symbols in place.
- .time_in_iso8601(time, fraction_digits = 3) ⇒ Object
-
.uid ⇒ Object
public: Returns a uid string.
Class Method Details
.convert_to_datetime(value) ⇒ Object
87 88 89 90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/posthog/utils.rb', line 87 def convert_to_datetime(value) if value.respond_to?(:strftime) value elsif value.respond_to?(:to_str) begin DateTime.parse(value) rescue ArgumentError raise InconclusiveMatchError, "#{value} is not in a valid date format" end else raise InconclusiveMatchError, 'The date provided must be a string or date object' end end |
.date_in_iso8601(date) ⇒ Object
73 74 75 |
# File 'lib/posthog/utils.rb', line 73 def date_in_iso8601(date) date.strftime('%F') end |
.datetime_in_iso8601(datetime) ⇒ Object
53 54 55 56 57 58 59 60 61 62 63 64 |
# File 'lib/posthog/utils.rb', line 53 def datetime_in_iso8601(datetime) case datetime when Time time_in_iso8601 datetime when DateTime time_in_iso8601 datetime.to_time when Date date_in_iso8601 datetime else datetime end end |
.formatted_offset(time, colon = true, alternate_utc_string = nil) ⇒ Object
77 78 79 80 |
# File 'lib/posthog/utils.rb', line 77 def formatted_offset(time, colon = true, alternate_utc_string = nil) (time.utc? && alternate_utc_string) || seconds_to_utc_offset(time.utc_offset, colon) end |
.is_valid_regex(regex) ⇒ Object
TODO: Rename to valid_regex? in future version
106 107 108 109 110 111 |
# File 'lib/posthog/utils.rb', line 106 def is_valid_regex(regex) # rubocop:disable Naming/PredicateName Regexp.new(regex) true rescue RegexpError false end |
.isoify_dates(hash) ⇒ Object
public: Returns a new hash with all the date values in the into iso8601
strings
31 32 33 34 35 |
# File 'lib/posthog/utils.rb', line 31 def isoify_dates(hash) hash.each_with_object({}) do |(k, v), memo| memo[k] = datetime_in_iso8601(v) end end |
.isoify_dates!(hash) ⇒ Object
public: Converts all the date values in the into iso8601 strings in place
39 40 41 |
# File 'lib/posthog/utils.rb', line 39 def isoify_dates!(hash) hash.replace isoify_dates hash end |
.seconds_to_utc_offset(seconds, colon = true) ⇒ Object
82 83 84 85 |
# File 'lib/posthog/utils.rb', line 82 def seconds_to_utc_offset(seconds, colon = true) format((colon ? UTC_OFFSET_WITH_COLON : UTC_OFFSET_WITHOUT_COLON), (seconds < 0 ? '-' : '+'), seconds.abs / 3600, (seconds.abs % 3600) / 60) end |
.stringify_keys(hash) ⇒ Object
public: Return a new hash with keys as strings
24 25 26 |
# File 'lib/posthog/utils.rb', line 24 def stringify_keys(hash) hash.each_with_object({}) { |(k, v), memo| memo[k.to_s] = v } end |
.symbolize_keys(hash) ⇒ Object
public: Return a new hash with keys converted from strings to symbols
12 13 14 |
# File 'lib/posthog/utils.rb', line 12 def symbolize_keys(hash) hash.each_with_object({}) { |(k, v), memo| memo[k.to_sym] = v } end |
.symbolize_keys!(hash) ⇒ Object
public: Convert hash keys from strings to symbols in place
18 19 20 |
# File 'lib/posthog/utils.rb', line 18 def symbolize_keys!(hash) hash.replace symbolize_keys hash end |
.time_in_iso8601(time, fraction_digits = 3) ⇒ Object
66 67 68 69 70 71 |
# File 'lib/posthog/utils.rb', line 66 def time_in_iso8601(time, fraction_digits = 3) fraction = (('.%06i' % time.usec)[0, fraction_digits + 1] if fraction_digits > 0) # rubocop:disable Style/FormatString "#{time.strftime('%Y-%m-%dT%H:%M:%S')}#{fraction}#{formatted_offset(time, true, 'Z')}" end |
.uid ⇒ Object
public: Returns a uid string
45 46 47 48 49 50 51 |
# File 'lib/posthog/utils.rb', line 45 def uid arr = SecureRandom.random_bytes(16).unpack('NnnnnN') arr[2] = (arr[2] & 0x0fff) | 0x4000 arr[3] = (arr[3] & 0x3fff) | 0x8000 '%08x-%04x-%04x-%04x-%04x%08x' % arr # rubocop:disable Style/FormatStringToken, Style/FormatString end |